<?php
$themeConfig = include __DIR__ . '/config.php';

/**
 * Transforms Content to Format:
 * yyyy
 *     {page},
 *     ...
 */
function transformContent($content)
{
    $transformedContent = [];
    foreach ($content as $page) {
        $dateEpoch = strtotime($page->dateRaw());
        $year = date('Y', $dateEpoch);
        $page->simpleDateFormat = date('M d', $dateEpoch);
        $transformedContent[$year][] = $page;
    }
    return $transformedContent;
}

function setItemsPerPage(int $count)
{
    global $site;
    if ($site->itemsPerPage() !== $count) {
        $site->set([
            'itemsPerPage' => $count
        ]);
    }
}

if ($themeConfig['enableIndexLayout']) {
    setItemsPerPage(-1);
} else {
    setItemsPerPage(8);
}
?>
<!DOCTYPE html>
<html lang="<?php echo Theme::lang() ?>">
<head>
<?php include(THEME_DIR_PHP . 'head.php'); ?>
</head>
<body>

    <?php
    // Load Bludit Plugins: Site Body Begin
    Theme::plugins('siteBodyBegin');

    // Navbar
    include(THEME_DIR_PHP . 'navbar.php');
    ?>

    <!-- Content -->
    <div class="container mt-2">
        <div class="row">

            <!-- Blog Posts -->
            <div class="col-md-9">
            <?php
            if ($WHERE_AM_I == 'page') {
                include(THEME_DIR_PHP . 'page.php');
            } else {
                // Index Layout
                if ($themeConfig['enableIndexLayout']) {
                    $transformedContent = transformContent($content);
                    foreach ($transformedContent as $year => $yearContent) {
                        echo "<h1>$year</h1>";
                        echo "<ul class='index-layout pl-0'>";
                        foreach ($yearContent as $page) {
                            echo "<li class='pb-2'>
                            {$page->simpleDateFormat}
                            <a href='{$page->permalink()}'>{$page->title()}</a>
                            </li>";
                        }
                        echo "</ul>";
                    }
                } else {
                    // Default Layout
                    include(THEME_DIR_PHP . 'home.php');
                }
            }
            ?>
            </div>

            <!-- Right Sidebar -->
            <div class="col-md-3">
            <?php Theme::plugins('siteSidebar'); ?>
            </div>

        </div>
    </div>

    <!-- Footer -->
    <footer class='bg-black text-light text-center text-uppercase'>
    <?= $site->footer() ?> | <a href="#" title="go-top">Go Top</a>
    </footer>

    <?php
    echo Theme::jquery();
    echo Theme::jsBootstrap();

    // Load Bludit Plugins: Site Body End
    Theme::plugins('siteBodyEnd');
    ?>

</body>
</html>
